import { Observable } from 'rxjs';
import { NgxHighlightConfig, HighlightResult, NgxHighlightOptions } from './highlight.model';
import { HighlightLoader } from './highlight.loader';
import { HLJSApi } from 'highlight.js';
import { Mode, HighlightOptions } from 'highlight.js';
export declare class HighlightService {
    private _loader;
    private _hljs;
    get hljs(): HLJSApi | null;
    constructor(_loader: HighlightLoader, options: NgxHighlightOptions);
    /**
     * Core highlighting function.
     * @param name Accepts a language name, or an alias
     * @param value A string with the code to highlight.
     * @param ignore_illegals When present and evaluates to a true value, forces highlighting to finish
     * even in case of detecting illegal syntax for the language instead of throwing an exception.
     * @param continuation An optional mode stack representing unfinished parsing.
     * When present, the function will restart parsing from this state instead of initializing a new one
     */
    highlight(codeOrLanguageName: string, optionsOrCode: string | HighlightOptions, ignoreIllegals?: boolean, continuation?: Mode): Observable<HighlightResult>;
    /**
     * Highlighting with language detection.
     * @param value Accepts a string with the code to highlight
     * @param languageSubset An optional array of language names and aliases restricting detection to only those languages.
     * The subset can also be set with configure, but the local parameter overrides the option if set.
     */
    highlightAuto(value: string, languageSubset: string[]): Observable<HighlightResult>;
    /**
     * Applies highlighting to a DOM node containing code.
     * The function uses language detection by default but you can specify the language in the class attribute of the DOM node.
     * See the class reference for all available language names and aliases.
     * @param block The element to apply highlight on.
     */
    highlightElement(block: HTMLElement): Observable<void>;
    /**
     * Configures global options:
     * @param config HighlightJs configuration argument
     */
    configure(config: NgxHighlightConfig): Observable<void>;
    /**
     * Applies highlighting to all <pre><code>..</code></pre> blocks on a page.
     */
    initHighlighting(): Observable<void>;
    /**
     * Adds new language to the library under the specified name. Used mostly internally.
     * @param name A string with the name of the language being registered
     * @param language A function that returns an object which represents the language definition.
     * The function is passed the hljs object to be able to use common regular expressions defined within it.
     */
    registerLanguage(name: string, language: () => any): Observable<HLJSApi>;
    /**
     * @return The languages names list.
     */
    listLanguages(): Observable<string[]>;
    /**
     * Looks up a language by name or alias.
     * @param name Language name
     * @return The language object if found, undefined otherwise.
     */
    getLanguage(name: string): Observable<any>;
    /**
     * Display line numbers
     * @param el Code element
     */
    lineNumbersBlock(el: HTMLElement): Observable<any>;
}
